"""
blink/cli/generate.py
Ian Kollipara
2022.04.02

Generate Command and subcommands
"""

# Imports
from datetime import date
from enum import Enum
from os.path import exists

from rich import get_console
from typer import Argument, Exit, Option, Typer

from .utils import render


class GenerationTypeEnum(str, Enum):
    """GenerationTypeEnum lists all possible choices for generating a type

    All of these are generated by Blink, and can be created at User descresion.
    """

    PAGE = "page"
    RESOURCE = "resource"
    MIDDLEWARE = "middleware"
    HOOK = "hook"
    MODEL = "model"


# These are standard for CLI tool
generate_cmd = Typer()
console = get_console()


@generate_cmd.command("generate")
def generate(
    g_type: GenerationTypeEnum = Argument(
        ..., help="Type of object to generate", metavar="TYPE"
    ),
    filename: str = Argument(
        ..., help="Name of File, should be snake_case", metavar="NAME"
    ),
    dry_run: bool = Option(
        False, "--dry-run", help="Show output, but don't actually create anything"
    ),
):
    """Generate the given type with the specified name.

    Given a valid TYPE and NAME, create a new module with that name. The corresponding
    class is then the PascalCased variant of that name.
    """

    default_params = {
        "filename": filename,
        "date": date.today().strftime("%Y.%m.%d"),
        "class_name": "".join(map(str.capitalize, filename.split("_"))),
    }

    match g_type:

        # Because the model type appends to the orm.models module,
        # there's no need to check if the file exists, as it has to for
        # Blink to work right. Given that, it needs its own special case.
        case GenerationTypeEnum.MODEL:
            rendered_template = render(
                "model.py.jinja", class_name=default_params["class_name"]
            )

            if not dry_run:
                with open("./orm/models.py", "a+", encoding="UTF-8") as model_file:
                    model_file.write(rendered_template)

            console.print(
                f"{default_params['class_name']} was successfully create at ./orm/models.py"
            )

        # All the other cases follow a similar definition, as such there's no need to
        # have a case for each specific type. 
        case _:
            rendered_template = render(f"{g_type.value}.py.jinja", **default_params)
            path = (
                f"./{g_type.value}/{filename}.py"
                if g_type != GenerationTypeEnum.HOOK
                else f"./hooks/{filename}.py"
            )

            if not dry_run:
                if exists(path):
                    console.print(f"[bold red]{filename} already exists!")
                    raise Exit(1)

                with open(path, "w+", encoding="UTF-8") as new_module:
                    new_module.write(rendered_template)

            console.print(f"[bold green] {filename} successfully created at {path}!")
