"""
blink/cli/manage.py
Ian Kollipara
2022.04.02

Manage Command Group
"""

# Imports
from importlib import import_module, util
from os import listdir
from os.path import isdir
from subprocess import run

from rich import get_console
from typer import Argument, Exit, Typer

manage_cmd = Typer()
console = get_console()


def is_module_installed(modname: str) -> bool:
    """Check if given module is available.

    Given a valid module name, check if the given module
    is present in the current environment. If not, return
    False otherwise True.
    """

    if util.find_spec(modname) is None:
        return False
    return True


@manage_cmd.command("format")
def blink_format(
    file_or_directory: str = Argument(".", help="File or Directory to format")
):
    """Format the file or directory using Black and Isort.

    Blink bundles Black and Isort to format your code. You must install Black and Isort
    yourself.
    """

    if not is_module_installed("black"):
        console.print(
            "[bold red]Black is not installed. Type[/red] [blue]poetry add --dev black"
        )
        raise Exit(1)

    if not is_module_installed("isort"):
        console.print(
            "[bold red]Isort is not installed. Type[/red] [blue]poetry add --dev isort"
        )
        raise Exit(1)


    run(["black", file_or_directory])
    run(["isort", file_or_directory])



@manage_cmd.command("lint")
def lint(file_or_directory: str = Argument(".", help="File or Directory to lint")):
    """Lint your code using Pylint.

    Blink bundles Pylint as its preferred linter. You must install pylint yourself.
    """

    if not is_module_installed("pylint"):
        console.print(
            "[bold red]Pylint is not installed. Type[/red] [blue]poetry add --dev pylint"
        )
        raise Exit(1)

    run(["pylint", file_or_directory], check=False)


@manage_cmd.command("test")
def test():
    """Test your code using Pytest.

    Blink bundles Pytest as its preferred testing suite. You must install pytest yourself.
    """

    if not is_module_installed("pytest"):
        console.print(
            "[bold red]Pytest is not installed. Type[/red] [blue]poetry add --dev pytest"
        )
        raise Exit(1)

    run(["pytest"], check=False)
