'''
blink/cli/new_cmd.py
Ian Kollipara
2022.04.03

New Project Generation Command
'''
# Imports
from typer import Typer, Argument, Option, Exit
from os.path import exists
from os import chdir, mkdir
from shutil import copy
from rich import get_console
from tomlkit import load
from datetime import date
from .utils import render


new_cmd = Typer()
console = get_console()


@new_cmd.command("new")
def new(proj_path: str = Argument(".", help="Path to initialize the project in.", metavar="PATH"), dry_run: bool = Option(False, "--dry-run", help="Show output, but don't create anything")):
    if not exists(proj_path):
        console.print("[bold red]Please initialize a Poetry project with either poetry new or poetry init.")
        Exit(1)
    
    chdir(proj_path)
    date_str = date.today().strftime("%Y.%m.%d")

    if not exists("pyproject.toml"):
        console.print("[bold red]Please initialize a Poetry project with poetry init.")
        Exit(1)
    
    poetry_config = load(open("pyproject.toml"))

    app_name: str = poetry_config["tool"]["poetry"]["name"] #type: ignore

    app_file = render("gen/app.py.jinja", app_name=app_name, date=date_str)
    config_file = render("gen/config.py.jinja", app_name=app_name, date=date_str)
    model_file = render("gen/models.py.jinja", date=date_str)
    page_file = render("page.py.jinja", filename="index.py", class_name="HomePage", date=date_str)
    resource_file = render("resource.py.jinja", filename="ping.py", class_name="Ping", date=date_str)

    if not dry_run:
        for dir in ("templates", "resources", "pages", "assets", "middleware", "hooks", "orm"):
            mkdir(dir)

        copy("templates/gen/htmx.min.js", "assets/htmx.min.js") 

        with open(f"{app_name}.py", "w+") as app:
            app.write(app_file)
        
        with open("config.py", "w+") as config:
            config.write(config_file)
        
        with open("orm/models.py", "w+") as orm:
            orm.write(model_file)
        
        with open("pages/index.py", "w+") as page:
            page.write(page_file)
        
        with open("resources/ping.py", "w+") as resource:
            resource.write(resource_file)
    
    console.print(f"[bold green]{app_name} was created! Have Fun!")
        